# API Documentation of Bash Base Library

## error messages

### error-codes

```
error-codes (bash-base)
#   1 .. unkown internal error
#   2 .. illegal number of arguments in function call
#   3 .. illegal variable name
#   4 .. illegal function arguments
#   5 .. error sourcing file
#   6 .. no such function
#   7 .. variable specified as argument undefined
#   8 .. no such file
#   9 .. no access rights
#  9x .. errors local to each function
```

### bb_error_string

```
bb_error_string <ret-vn> <error-code>
# errors: 2, 3
```

## generic tools

### bb_validate_vn

```
bb_validate_vn <caller-name> <vn> [..]
# validates one or more variable names
# valid variable names match the following regex: ^[[:alpha:]][[:alnum:]_]*$
# errors: 2, 3
```

## array tools

### bb_array_iter

```
bb_array_iter <array-vn> <iterfun-name> [<arg> [..]]
# iter-fun takes 2 args: <ret-vn> <input-value>
# if target array is not empty contents will be merged in
# errors: 2, 3, 8
```

### bb_array_map

```
bb_array_map <ret-vn> <src-vn> <mapfun-name> [<arg> [..]]
# map-fun takes 2 args: <ret-vn> <input-value>
# if target array is not empty contents will be merged in
# errors: 2, 3, 8
```

### bb_array_filter

```
bb_array_filter <ret-vn> <src-vn> <filterfun-name> [<arg> [..]]
# filterfun takes 1 arg <value> and shall return 0 if value shell be accepted
# errors: 2, 3, 8
```

### bb_array_copy

```
bb_array_copy <dst-vn> <src-vn>
# copys an array, target array must exist and if non empty will be merged
# errors: same as bb_array_map
```

### bb_array_length

```
bb_array_length <ret-vn> <array-vn>
# returns the length of an array
# errors: 2, 3
```

### bb_mapfun_identity

```
bb_mapfun_identity <dst-vn> <input-value>
# the identity fun
# errors: 2, 3
```

### bb_array_join_by

```
bb_array_join_by <ret-vn> <join-string> [<element0> [..]]
# joins array element with join-string, returns "" if no elements specified
# errors: 2, 3
```

## error handling: handle internal errors in a configurable way

### bb_ierr

```
bb_ierr <msg> [..]
# call this on internal error, depending on mode it does:
#  mode "fatal": pass msg to bb_fatal
#  other mode: print error msg with specified loglevel
```

### bb_ierr_set_mode

```
bb_ierr_set_mode [fatal|<msg-loglevel>]
# default is "err"
```

### bb_fatal

```
bb_fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
# since it produces some lines of output it is meant for internal errors
# only, if you want to abort your script with a message use bb_quit
```

## msg: log/show messages

### bb_msg_levelname_of_string

```
bb_msg_levelname_of_string <ret-vn> <string>
# returns level as string if input was integer
# else returns string if it is a valid level
# errors: 2, 3, 4
```

### bb_msg_int_of_level

```
bb_msg_int_of_level
# convert level name to int or return int unmodified if in valid range
# errors: 2, 3, 4
```

### bb_msg_set_target

```
bb_msg_set_target <target-name> <type>[:<arg>] <min-level>[:<max-level]
#                   <prefix> [[no]trace]
# where type can be one of: stdout, stderr, file, syslog
# and arg is filename for file, facility for syslog
# level is one of: debug, info, notice, warning, err, crit, alert, emerg
# prefix can be: none, timestamp, prog, progpid, full
# if notrace is specified, no stack traces will be print to that target
# notrace is the default
# note: target-name must match regex ^[[:alpha:]][[:alnum:]_]*$
# errors: 2, 4
```

### bb_msg_del_target

```
bb_msg_del_target <target-name>
# errors: 2
```

### bb_msg_clear_targets

```
bb_msg_clear_targets
# remove all message targets
# errors: 2
# shellcheck disable=SC2120
```

### bb_msg

```
bb_msg <level> <string> [..]
# logs messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msgf

```
bb_msgf <level> <format> [<format-arg> [..]]
# logs printf style messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msg_stacktrace

```
bb_msg_stacktrace <ret-vn> [<skip>]
# create a human readable stacktrace, excluding this function and
# returns it as array of strings
# optional arg skip can be used to manually configure how many inner
# levels will not be displayed
# errors: 2, 3
```

## log functions (deprecated, use msg instead)

### bb_logf

```
bb_logf <message> [..]
# logs message file name in $LOG_FILE if non zero
```

### bb_log

```
bb_log <message> [..]
# logs message to stderr and if define to file name in $LOG_FILE
```

### bb_log_debug

```
bb_log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
# errors: 2
```

## flow control and error handling

### bb_finalize

```
bb_finalize
# unregisters all traps
```

### bb_quit

```
bb_quit [<exit-code> [<log message>] [..]]
# the safe way to clean exit a script using bash_base
```

### bb_exec

```
bb_exec <cmd> [arg [..]]
# execs cmd with args if specified
```

### bb_traps_setup

```
bb_traps_setup
# will be removed in the future
```

### bb_traps_disable

```
bb_traps_disable
# will be removed in the future
```

### bb_version

```
bb_version <ret-vn>
# returns the version of bash base
# errors: 2, 3
```

### bb_register_exit_hook

```
bb_register_exit_hook [ <hook-name> ] <function-name>
# registers a function to be called on exit, it is not possible to
# pass arguments to that function
# hooks will be called sorted by hooknames
# note: the default hook-name is the function-name
```

### bb_unregister_exit_hook

```
bb_unregister_exit_hook <hookn-ame>
# removes a registered exit hook
# errors: 2
```

### bb_load_lib

```
bb_load_lib <name>
# sources file for lib specified
# if name is no absolute or local path it tries:
# /usr/libexec/<name>.bash
# /usr/libexec/<name>/<name>.bash
# errors: 2, 5, 8, 91 (lib not found)
```

### bb_load_libs

```
bb_load_libs <name> [..]
# load one or more libs but continue if some fail to load
# errors: 2, 91 (not all libs loaded)
```

### bb_require_libs

```
bb_require_libs <name> [..]
# load one or more libs and raise fatal error if any fail
# errors: 2, bb_fatal on any other error
```
